"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertAdditionalProperties = exports.convert = exports.generateMultiTypeNode = void 0;
const Exception_1 = require("../Exception");
const Logger = __importStar(require("../Logger"));
const Reference = __importStar(require("./components/Reference"));
const Guard = __importStar(require("./Guard"));
const InferredType = __importStar(require("./InferredType"));
const generateMultiTypeNode = (entryPoint, currentPoint, factory, schemas, setReference, convert, convertContext, multiType) => {
    const typeNodes = schemas.map(schema => convert(entryPoint, currentPoint, factory, schema, setReference, convertContext));
    if (multiType === "oneOf") {
        return factory.UnionTypeNode.create({
            typeNodes,
        });
    }
    if (multiType === "allOf") {
        return factory.IntersectionTypeNode.create({
            typeNodes,
        });
    }
    /**
     * If you see this comment and have an idea for an AnyOf type output, please submit an Issue.
     */
    return factory.UnionTypeNode.create({
        typeNodes,
    });
};
exports.generateMultiTypeNode = generateMultiTypeNode;
const nullable = (factory, typeNode, nullable) => {
    if (nullable) {
        return factory.UnionTypeNode.create({
            typeNodes: [
                typeNode,
                factory.TypeNode.create({
                    type: "null",
                }),
            ],
        });
    }
    return typeNode;
};
const convert = (entryPoint, currentPoint, factory, schema, context, converterContext, option) => {
    if (typeof schema === "boolean") {
        // https://swagger.io/docs/specification/data-models/dictionaries/#free-form
        return factory.TypeNode.create({
            type: "object",
            value: [],
        });
    }
    if (Guard.isReference(schema)) {
        const reference = Reference.generate(entryPoint, currentPoint, schema);
        if (reference.type === "local") {
            // Type Aliasを作成 (or すでにある場合は作成しない)
            context.setReferenceHandler(currentPoint, reference);
            const { maybeResolvedName, depth, pathArray } = context.resolveReferencePath(currentPoint, reference.path);
            if (depth === 2) {
                return factory.TypeReferenceNode.create({ name: converterContext.escapeReferenceDeclarationText(maybeResolvedName) });
            }
            else {
                const resolveSchema = context.findSchemaByPathArray(pathArray);
                return exports.convert(entryPoint, currentPoint, factory, resolveSchema, context, converterContext, { parent: schema });
            }
        }
        // サポートしているディレクトリに対して存在する場合
        if (reference.componentName) {
            // Type AliasもしくはInterfaceを作成
            context.setReferenceHandler(currentPoint, reference);
            // Aliasを貼る
            return factory.TypeReferenceNode.create({ name: context.resolveReferencePath(currentPoint, reference.path).name });
        }
        // サポートしていないディレクトリに存在する場合、直接Interface、もしくはTypeAliasを作成
        return exports.convert(entryPoint, reference.referencePoint, factory, reference.data, context, converterContext, { parent: schema });
    }
    if (Guard.isOneOfSchema(schema)) {
        return nullable(factory, exports.generateMultiTypeNode(entryPoint, currentPoint, factory, schema.oneOf, context, exports.convert, converterContext, "oneOf"), !!schema.nullable);
    }
    if (Guard.isAllOfSchema(schema)) {
        return nullable(factory, exports.generateMultiTypeNode(entryPoint, currentPoint, factory, schema.allOf, context, exports.convert, converterContext, "allOf"), !!schema.nullable);
    }
    if (Guard.isAnyOfSchema(schema)) {
        return nullable(factory, exports.generateMultiTypeNode(entryPoint, currentPoint, factory, schema.anyOf, context, exports.convert, converterContext, "anyOf"), !!schema.nullable);
    }
    if (Guard.isHasNoMembersObject(schema)) {
        return factory.TypeNode.create({
            type: "object",
            value: [],
        });
    }
    // schema.type
    if (!schema.type) {
        const inferredSchema = InferredType.getInferredType(schema);
        if (inferredSchema) {
            return exports.convert(entryPoint, currentPoint, factory, inferredSchema, context, converterContext, { parent: schema });
        }
        // typeを指定せずに、nullableのみを指定している場合に type object変換する
        if (typeof schema.nullable === "boolean") {
            const typeNode = factory.TypeNode.create({
                type: "any",
            });
            return nullable(factory, typeNode, schema.nullable);
        }
        if (option && option.parent) {
            const message = [
                "Schema Type is not found and is converted to the type any. The parent Schema is as follows.",
                "",
                JSON.stringify(option.parent),
                "",
            ].join("\n");
            Logger.info(message);
        }
        const typeNode = factory.TypeNode.create({
            type: "any",
        });
        return typeNode;
        // Logger.showFilePosition(entryPoint, currentPoint);
        // throw new UnsetTypeError("Please set 'type' or '$ref' property \n" + JSON.stringify(schema));
    }
    switch (schema.type) {
        case "boolean": {
            const items = schema.enum;
            let typeNode;
            if (items && Guard.isBooleanArray(items)) {
                typeNode = factory.TypeNode.create({
                    type: schema.type,
                    enum: items,
                });
            }
            else {
                typeNode = factory.TypeNode.create({
                    type: schema.type,
                });
            }
            return nullable(factory, typeNode, !!schema.nullable);
        }
        case "null": {
            return factory.TypeNode.create({
                type: schema.type,
            });
        }
        case "integer":
        case "number": {
            const items = schema.enum;
            let typeNode;
            const formatTypeNode = converterContext.convertFormatTypeNode(schema);
            if (formatTypeNode) {
                return formatTypeNode;
            }
            if (items && Guard.isNumberArray(items)) {
                typeNode = factory.TypeNode.create({
                    type: schema.type,
                    enum: items,
                });
            }
            else {
                typeNode = factory.TypeNode.create({
                    type: schema.type,
                });
            }
            return nullable(factory, typeNode, !!schema.nullable);
        }
        case "string": {
            const items = schema.enum;
            const formatTypeNode = converterContext.convertFormatTypeNode(schema);
            if (formatTypeNode) {
                return formatTypeNode;
            }
            let typeNode;
            if (items && Guard.isStringArray(items)) {
                typeNode = factory.TypeNode.create({
                    type: schema.type,
                    enum: items,
                });
            }
            else {
                typeNode = factory.TypeNode.create({
                    type: schema.type,
                });
            }
            return nullable(factory, typeNode, !!schema.nullable);
        }
        case "array": {
            if (Array.isArray(schema.items) || typeof schema.items === "boolean") {
                throw new Exception_1.UnSupportError(`schema.items = ${JSON.stringify(schema.items)}`);
            }
            const typeNode = factory.TypeNode.create({
                type: schema.type,
                value: schema.items
                    ? exports.convert(entryPoint, currentPoint, factory, schema.items, context, converterContext, { parent: schema })
                    : factory.TypeNode.create({
                        type: "undefined",
                    }),
            });
            return nullable(factory, typeNode, !!schema.nullable);
        }
        case "object": {
            const required = schema.required || [];
            // // https://swagger.io/docs/specification/data-models/dictionaries/#free-form
            if (schema.additionalProperties === true) {
                return factory.TypeNode.create({
                    type: schema.type,
                    value: [],
                });
            }
            const value = Object.entries(schema.properties || {}).map(([name, jsonSchema]) => {
                return factory.PropertySignature.create({
                    name: converterContext.escapePropertySignatureName(name),
                    type: exports.convert(entryPoint, currentPoint, factory, jsonSchema, context, converterContext, { parent: schema.properties }),
                    optional: !required.includes(name),
                    comment: typeof jsonSchema !== "boolean" ? jsonSchema.description : undefined,
                });
            });
            if (schema.additionalProperties) {
                const additionalProperties = factory.IndexSignatureDeclaration.create({
                    name: "key",
                    type: exports.convert(entryPoint, currentPoint, factory, schema.additionalProperties, context, converterContext, {
                        parent: schema.properties,
                    }),
                });
                const hasOptionalProperty = Object.keys(schema.properties || {}).some(key => !required.includes(key));
                if (hasOptionalProperty) {
                    const objectTypeNode = factory.TypeNode.create({
                        type: schema.type,
                        value: value,
                    });
                    const additionalObjectTypeNode = factory.TypeNode.create({
                        type: schema.type,
                        value: [additionalProperties],
                    });
                    return factory.IntersectionTypeNode.create({
                        typeNodes: [objectTypeNode, additionalObjectTypeNode],
                    });
                }
                return factory.TypeNode.create({
                    type: schema.type,
                    value: [...value, additionalProperties],
                });
            }
            const typeNode = factory.TypeNode.create({
                type: schema.type,
                value,
            });
            return nullable(factory, typeNode, !!schema.nullable);
        }
        default:
            return factory.TypeNode.create({
                type: "any",
            });
        // throw new UnknownError("what is this? \n" + JSON.stringify(schema, null, 2));
    }
};
exports.convert = convert;
const convertAdditionalProperties = (entryPoint, currentPoint, factory, schema, setReference, convertContext) => {
    // // https://swagger.io/docs/specification/data-models/dictionaries/#free-form
    if (schema.additionalProperties === true) {
        factory.TypeNode.create({
            type: schema.type,
            value: [],
        });
    }
    const additionalProperties = factory.IndexSignatureDeclaration.create({
        name: "key",
        type: exports.convert(entryPoint, currentPoint, factory, schema.additionalProperties, setReference, convertContext, { parent: schema.properties }),
    });
    return additionalProperties;
};
exports.convertAdditionalProperties = convertAdditionalProperties;
//# sourceMappingURL=toTypeNode.js.map