"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.make = exports.create = void 0;
const typescript_1 = __importDefault(require("typescript"));
const Exception_1 = require("../../Exception");
const LiteralTypeNode = __importStar(require("./LiteralTypeNode"));
const UnionTypeNode = __importStar(require("./UnionTypeNode"));
const create = (context) => (params) => {
    const { factory } = context;
    const literalTypeNode = LiteralTypeNode.create(context);
    const unionTypeNode = UnionTypeNode.create(context);
    const createNode = () => {
        switch (params.type) {
            case "string":
                if (params.enum) {
                    return unionTypeNode({
                        typeNodes: params.enum.map(value => literalTypeNode({ value })),
                    });
                }
                return factory.createKeywordTypeNode(typescript_1.default.SyntaxKind.StringKeyword);
            case "number":
            case "integer":
                if (params.enum) {
                    return unionTypeNode({
                        typeNodes: params.enum.map(value => literalTypeNode({ value })),
                    });
                }
                return factory.createKeywordTypeNode(typescript_1.default.SyntaxKind.NumberKeyword);
            case "boolean": {
                if (params.enum) {
                    return unionTypeNode({
                        typeNodes: params.enum.map(value => literalTypeNode({ value })),
                    });
                }
                return factory.createKeywordTypeNode(typescript_1.default.SyntaxKind.BooleanKeyword);
            }
            case "object":
                return factory.createTypeLiteralNode(params.value);
            case "undefined":
                return factory.createKeywordTypeNode(typescript_1.default.SyntaxKind.UndefinedKeyword);
            case "null":
                return factory.createLiteralTypeNode(factory.createNull());
            case "array":
                return factory.createArrayTypeNode(params.value);
            case "never":
                return factory.createKeywordTypeNode(typescript_1.default.SyntaxKind.NeverKeyword);
            case "void":
                return factory.createKeywordTypeNode(typescript_1.default.SyntaxKind.VoidKeyword);
            case "any":
                return factory.createKeywordTypeNode(typescript_1.default.SyntaxKind.AnyKeyword);
            default:
                throw new Exception_1.UnSupportError(`UnSupport Type: ${JSON.stringify(params)}`);
        }
    };
    const node = createNode();
    return node;
};
exports.create = create;
const make = (context) => {
    return {
        create: exports.create(context),
    };
};
exports.make = make;
//# sourceMappingURL=TypeNode.js.map