import ts from "typescript";
import { UnSupportError } from "../../Exception";
import * as LiteralTypeNode from "./LiteralTypeNode";
import * as UnionTypeNode from "./UnionTypeNode";
export const create = (context) => (params) => {
    const { factory } = context;
    const literalTypeNode = LiteralTypeNode.create(context);
    const unionTypeNode = UnionTypeNode.create(context);
    const createNode = () => {
        switch (params.type) {
            case "string":
                if (params.enum) {
                    return unionTypeNode({
                        typeNodes: params.enum.map(value => literalTypeNode({ value })),
                    });
                }
                return factory.createKeywordTypeNode(ts.SyntaxKind.StringKeyword);
            case "number":
            case "integer":
                if (params.enum) {
                    return unionTypeNode({
                        typeNodes: params.enum.map(value => literalTypeNode({ value })),
                    });
                }
                return factory.createKeywordTypeNode(ts.SyntaxKind.NumberKeyword);
            case "boolean": {
                if (params.enum) {
                    return unionTypeNode({
                        typeNodes: params.enum.map(value => literalTypeNode({ value })),
                    });
                }
                return factory.createKeywordTypeNode(ts.SyntaxKind.BooleanKeyword);
            }
            case "object":
                return factory.createTypeLiteralNode(params.value);
            case "undefined":
                return factory.createKeywordTypeNode(ts.SyntaxKind.UndefinedKeyword);
            case "null":
                return factory.createLiteralTypeNode(factory.createNull());
            case "array":
                return factory.createArrayTypeNode(params.value);
            case "never":
                return factory.createKeywordTypeNode(ts.SyntaxKind.NeverKeyword);
            case "void":
                return factory.createKeywordTypeNode(ts.SyntaxKind.VoidKeyword);
            case "any":
                return factory.createKeywordTypeNode(ts.SyntaxKind.AnyKeyword);
            default:
                throw new UnSupportError(`UnSupport Type: ${JSON.stringify(params)}`);
        }
    };
    const node = createNode();
    return node;
};
export const make = (context) => {
    return {
        create: create(context),
    };
};
//# sourceMappingURL=TypeNode.js.map