import { Tree } from "@himenon/path-oriented-data-structure";
import ts from "typescript";
import type { OpenApi } from "../../../types";
import { Factory } from "../../TsGenerator";
import * as Def from "./Definition";
import * as Operation from "./Operation";
import * as State from "./State";
import * as Structure from "./structure";
export interface AddStatementOption {
    /**
     * pathに対して強制的にSchemaを上書きするフラグ
     * TypeAliasが先に登録され、Primitiveな型定義が登録されない問題を解決する
     */
    override?: boolean;
}
declare class Store {
    private factory;
    private state;
    private operator;
    private getChildByPaths;
    constructor(factory: Factory.Type, rootDocument: OpenApi.Document);
    convertNamespace(tree: Tree<Structure.NamespaceTree.Kind> | Structure.NamespaceTree.Item): ts.Statement;
    private capitalizeFirstLetter;
    getRootStatements(): ts.Statement[];
    getAdditionalStatements(): ts.Statement[];
    /**
     * @params path: "components/headers/hoge"
     */
    hasStatement(path: string, types: Structure.DataStructure.Kind[]): boolean;
    /**
     * @params path: "components/headers/hoge"
     */
    addStatement(path: string, statement: Structure.ComponentParams, options?: AddStatementOption): void;
    getStatement<T extends Structure.DataStructure.Kind>(path: string, kind: T): Structure.DataStructure.GetChild<T> | undefined;
    addComponent(componentName: Def.ComponentName, statement: Structure.ComponentParams): void;
    getNoReferenceOperationState(): Operation.State;
    updateOperationState(httpMethod: string, requestUri: string, operationId: string, newOperationState: Partial<State.OperationState>): void;
    addAdditionalStatement(statements: ts.Statement[]): void;
    getPathItem(localPath: string): OpenApi.PathItem;
    getParameter(localPath: string): OpenApi.Parameter;
    isAfterDefined(referencePath: string): boolean;
}
export { Store };
